VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "KeyboardIO"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

'==================================================================
'
' Keyboard IO Class
'
' 30 September 2001
' 09 March 2002 - Updated RaiseKeyEvent
'
' You're free in using this software, as long as the
' orignal copyright/author information remains intact.
' In other words, respecting the person(s) who made it
' possible for you to view/learn something of this software.
'
' Copyright  2001 - Diederik van der Boor
' http://www.codingdomain.com
'
'==================================================================


'------------------------------------------------------------------
' Public Enums

Public Enum KeyEventRange
  keyDesktopWindow = 0
  keyActiveWindow = 1
End Enum

Private Const KEYEVENTF_EXTENDEDKEY = &H1
Private Const KEYEVENTF_KEYUP = &H2

Public Enum KeyboardKeys
' Virtual Keys, Standard Set
  VK_LBUTTON = &H1
  VK_RBUTTON = &H2
  VK_CANCEL = &H3
  VK_MBUTTON = &H4             '  NOT contiguous with L RBUTTON
  
  VK_BACK = &H8
  VK_TAB = &H9
  
  VK_CLEAR = &HC
  VK_RETURN = &HD
  
  VK_SHIFT = &H10
  VK_CONTROL = &H11
  VK_MENU = &H12
  VK_PAUSE = &H13
  VK_CAPITAL = &H14
  
  VK_ESCAPE = &H1B
  
  VK_SPACE = &H20
  VK_PRIOR = &H21
  VK_NEXT = &H22
  VK_END = &H23
  VK_HOME = &H24
  VK_LEFT = &H25
  VK_UP = &H26
  VK_RIGHT = &H27
  VK_DOWN = &H28
  VK_SELECT = &H29
  VK_PRINT = &H2A
  VK_EXECUTE = &H2B
  VK_SNAPSHOT = &H2C
  VK_INSERT = &H2D
  VK_DELETE = &H2E
  VK_HELP = &H2F
  
' VK_A thru VK_Z are the same as their ASCII equivalents: 'A' thru 'Z'
' VK_0 thru VK_9 are the same as their ASCII equivalents: '0' thru '9'
  
  VK_NUMPAD0 = &H60
  VK_NUMPAD1 = &H61
  VK_NUMPAD2 = &H62
  VK_NUMPAD3 = &H63
  VK_NUMPAD4 = &H64
  VK_NUMPAD5 = &H65
  VK_NUMPAD6 = &H66
  VK_NUMPAD7 = &H67
  VK_NUMPAD8 = &H68
  VK_NUMPAD9 = &H69
  VK_MULTIPLY = &H6A
  VK_ADD = &H6B
  VK_SEPARATOR = &H6C
  VK_SUBTRACT = &H6D
  VK_DECIMAL = &H6E
  VK_DIVIDE = &H6F
  VK_F1 = &H70
  VK_F2 = &H71
  VK_F3 = &H72
  VK_F4 = &H73
  VK_F5 = &H74
  VK_F6 = &H75
  VK_F7 = &H76
  VK_F8 = &H77
  VK_F9 = &H78
  VK_F10 = &H79
  VK_F11 = &H7A
  VK_F12 = &H7B
  VK_F13 = &H7C
  VK_F14 = &H7D
  VK_F15 = &H7E
  VK_F16 = &H7F
  VK_F17 = &H80
  VK_F18 = &H81
  VK_F19 = &H82
  VK_F20 = &H83
  VK_F21 = &H84
  VK_F22 = &H85
  VK_F23 = &H86
  VK_F24 = &H87
  
  VK_NUMLOCK = &H90
  VK_SCROLL = &H91
  
'   VK_L VK_R - left and right Alt, Ctrl and Shift virtual keys.
'   Used only as parameters to GetAsyncKeyState() and GetKeyState().
'   No other API or message will distinguish left and right keys in this way.
  VK_LSHIFT = &HA0
  VK_RSHIFT = &HA1
  VK_LCONTROL = &HA2
  VK_RCONTROL = &HA3
  VK_LMENU = &HA4
  VK_RMENU = &HA5
  
  VK_ATTN = &HF6
  VK_CRSEL = &HF7
  VK_EXSEL = &HF8
  VK_EREOF = &HF9
  VK_PLAY = &HFA
  VK_ZOOM = &HFB
  VK_NONAME = &HFC
  VK_PA1 = &HFD
  VK_OEM_CLEAR = &HFE
End Enum



'------------------------------------------------------------------
' API Declarations

Private Const KEYS_NUM = 255

Private Declare Sub keybd_event Lib "user32" (ByVal bVk As Byte, ByVal bScan As Byte, ByVal dwFlags As Long, ByVal dwExtraInfo As Long)
Private Declare Function GetKeyboardState Lib "user32" (pbKeyState As Byte) As Long
Private Declare Function SetKeyboardState Lib "user32" (lppbKeyState As Byte) As Long


'------------------------------------------------------------------
' Is a keypressed?

Public Property Get KeyPressed(Key As KeyboardKeys, Optional ByVal KeyList As String) As Boolean
  If KeyList = "" Then
    KeyList = Me.KeysPressed
  ElseIf Len(KeyList) <> KEYS_NUM Then
    Err.Raise 380 ' Inv Prop Value
  End If
  KeyPressed = Mid(KeyList, Key + 1, 1)
End Property

Public Property Let KeyPressed(Key As KeyboardKeys, Optional ByVal KeyList As String, NewState As Boolean)
  If KeyList = "" Then KeyList = Me.KeysPressed
  KeyList = Left(KeyList, Key) & IIf(NewState, 1, 0) & Mid(KeyList, Key + 2)
  Me.KeysPressed = KeyList
End Property


'------------------------------------------------------------------
' All the keys pressed in a string array.
' These methods can be called within VB5 aswell,
' since VB5 can't return arrays!!!

Public Property Get KeysPressed() As String
  Dim Key As Variant
  Dim Keys(1 To KEYS_NUM) As Byte
  
  If GetKeyboardState(Keys(1)) = 0 Then
    Err.Raise 12345, "Keyboard", "Could not read key states!"
  Else
    For Each Key In Keys
      KeysPressed = KeysPressed & IIf(Key <> 0, 1, 0)
    Next
  End If
End Property

Public Property Let KeysPressed(ByVal NewValue As String)
  Dim I As Byte
  Dim Keys(1 To KEYS_NUM) As Byte
  
  If Len(NewValue) <> KEYS_NUM Then Err.Raise 380 ' Inv Prop Value
  For I = 0 To KEYS_NUM - 1
    Keys(I + 1) = Mid(NewValue, I + 1, 1)
  Next
  
  If SetKeyboardState(Keys(1)) = 0 Then _
    Err.Raise 12345, "Keyboard", "Could not set key states!"
End Property

'Exx: Call KeyCreateEvent(VK_SNAPSHOT, keyDesktopWindow, 0&, 0&)
Public Sub RaiseKeyEvent(Key As KeyboardKeys, IsReleased As Boolean, Optional IsExtended As Boolean, Optional Range As KeyEventRange = keyDesktopWindow, Optional ExtraInfo As Long)
  Dim Flags As Long
  If IsExtended Then Flags = Flags Or KEYEVENTF_EXTENDEDKEY
  If IsReleased Then Flags = Flags Or KEYEVENTF_KEYUP
  Call keybd_event(Key, CByte(Range), Flags, ExtraInfo)
End Sub

